function RaidExtension.Data.Update()
	local keys = {}
	RaidExtension.Data.UnitList = {}

	for i = 1, 21 do
		local unitDetail = Inspect.Unit.Detail(iif(i == 21, "player", string.format("group%.2d", i)))

		if unitDetail then
			if keys[unitDetail.name] == nil then
				table.insert(RaidExtension.Data.UnitList, unitDetail.name)
				keys[unitDetail.name] = true
				local unitData = RaidExtension.Data.GetUnit(unitDetail.name)

				for k, v in pairs(unitData) do
					if k:find("buff.") == 1 then unitData[k] = nil end
				end
				unitData["bufflist.enchant"] = {}
				unitData["bufflist.food"] = {}
				unitData["bufflist.vial"] = {}

				unitData["props.class"] = RaidExtension.Utils.ParseClass(unitDetail.calling)
				unitData["props.role"] = unitDetail.role
				unitData["props.planar"] = ifnull(unitDetail.planar, 0)
				unitData["props.planarMax"] = ifnull(unitDetail.planarMax, 0)
				unitData["props.vitality"] = ifnull(unitDetail.vitality, 0)
				unitData["props.health"] = ifnull(unitDetail.healthMax, 0)
				unitData["props.mana"] = ifnull(unitDetail.manaMax, 0)

				local unitBuff = Inspect.Buff.List(unitDetail.id)
				if unitBuff then
					for k, v in pairs(unitBuff) do
						local buffDetail = Inspect.Buff.Detail(unitDetail.id, k)

						if RaidExtension.Data.BuffList[buffDetail.name] ~= nil then
							local buff = RaidExtension.Data.BuffList[buffDetail.name]
							if RaidExtension.Data.BuffList[buffDetail.name].value ~= nil then
								if unitData["buff." .. buff.category .. ".value"] == nil then
									unitData["buff." .. buff.category .. ".resistance"] = buff.resistance
									unitData["buff." .. buff.category .. ".icon"] = buffDetail.icon
									unitData["buff." .. buff.category .. ".remaining"] = buffDetail.remaining
									unitData["buff." .. buff.category .. ".value"] = buff.value
									unitData["buff." .. buff.category .. ".short"] = buff.short
									unitData["buff." .. buff.category .. ".quality"] = buff.quality
								else
									unitData["buff." .. buff.category .. ".remaining"] = math.min(unitData["buff." .. buff.category .. ".remaining"], buffDetail.remaining)
									unitData["buff." .. buff.category .. ".value"] = unitData["buff." .. buff.category .. ".value"] + buff.value
									unitData["buff." .. buff.category .. ".quality"] = math.max(unitData["buff." .. buff.category .. ".quality"], buff.quality)
								end
							end
							table.insert(unitData["bufflist." .. buff.category], { description = buff.description, icon = buffDetail.icon, remaining = buffDetail.remaining })
						end
					end
				end
			end
		end
	end

	for i = 1, 20 do
		RaidExtension.Storage.RetrieveData(RaidExtension.Data.UnitList[i])
	end
end

function RaidExtension.Data.GetUnit(id)
	if RaidExtension.Data.Units[id] == nil then
		RaidExtension.Data.Units[id] = { name = id }
	end
	return RaidExtension.Data.Units[id]
end

function RaidExtension.Data.OnReceiveData(target, data)
	local unitData = RaidExtension.Data.GetUnit(target)

	for k, v in pairs(unitData) do
		if k:find("data.") == 1 then unitData[k] = nil end
	end

	for k, v in pairs(data) do
		unitData["data." .. k] = v
	end
end

function RaidExtension.Data.Sort(field, direction)
	table.sort(RaidExtension.Data.UnitList, function(arg1, arg2) return RaidExtension.Data.OnSort(field, direction, arg1, arg2) end)
	return RaidExtension.Data.UnitList
end

function RaidExtension.Data.OnSort(field, direction, arg1, arg2)
	local val1, val2 = arg1, arg2
	local status

	if field then
		val1 = ifnull(RaidExtension.Data.Units[arg1][field], "0")
		val2 = ifnull(RaidExtension.Data.Units[arg2][field], "0")
	end

	if tonumber(val1) ~= nil then val1 = tonumber(val1) end
	if tonumber(val2) ~= nil then val2 = tonumber(val2) end

	return iif(direction, val1 < val2, val1 > val2)
end
