function RaidExtension.EGUI.CreateRow(parent, id, columns)
	local frame = RaidExtension.GUI.CreateFrame(parent, "TOPLEFT", 0, id * 20 - 20, 890, 19, 0)
	frame.role = RaidExtension.GUI.CreateTexture(frame, 2, 2, 16, 16, 1)
	RaidExtension.GUI.CreateTexture(frame, 2, 2, 15, 15, 2, "img/icon-border.png")
	frame.Fields = {}
	frame.background = 0.05

	for k, v in ipairs(columns) do
		if v.fieldType == 0 then
			frame.Fields[k] = RaidExtension.GUI.CreateText(frame, v.fieldAlign, v.fieldX, -1)
		else
			frame.Fields[k] = RaidExtension.GUI.CreateIconicText(frame, v.fieldAlign, v.fieldX, -1, v.fieldWidth, 19, v.fieldType == 2)
		end
		frame.Fields[k].obj = v
	end

	function frame.Update(data)
		if data then
			frame.data = data
			frame.background = iif(data.name == Inspect.Unit.Detail("player").name, 0.15, 0.05)
			frame.Show()
			if data["props.role"] then
				frame.role:SetTexture("Rift", "raid_icon_role_" .. data["props.role"] .. ".png.dds")
			end
			frame.role:SetVisible(data["props.role"] ~= nil)

			for k, v in ipairs(frame.Fields) do
				val = RaidExtension.Data.Units[data.name][v.obj.fieldName]
				if val ~= nil then
					if v.obj.draw then v.obj.draw(v, data) else v:SetText(tostring(val)) end
				end
				v:SetVisible(val ~= nil)
			end
		end
		frame:SetVisible(data ~= nil)
	end

	function frame.Event:MouseIn() frame.hover = true frame.Show() end
	function frame.Event:MouseOut() frame.hover = false frame.Show() end
	function frame.Event:MouseMove() frame.Show() end
	function frame.Show() 
		frame:SetBackgroundColor(1, 1, 1, frame.background + iif(frame.hover, .1, 0)) 
		if frame.hover and frame.data["props.class"] ~= 1 then RaidExtension.Tooltip.Show(id, parent:GetLeft() + iif(parent:GetLeft() + parent:GetWidth() / 2 < UIParent:GetWidth() / 2, parent:GetWidth(), -320), parent:GetTop() + id * 20 - 24, frame.data, Inspect.Mouse().x - parent:GetLeft() < 390) end
		if not frame.hover then RaidExtension.Tooltip.Hide(id) end
	end

	return frame
end

function RaidExtension.EGUI.AttachDragFrame(parent, fTitle, variable, fLeft)
	local frame = nil
	if fTitle then
		frame = RaidExtension.GUI.CreateFrame(parent:GetBorder(), "TOPLEFT", 0, 0, parent:GetBorder():GetWidth(), 45, 100)
	else
		frame = RaidExtension.GUI.CreateFrame(parent, "TOPLEFT", 0, 0, parent:GetWidth(), parent:GetHeight(), 100)
	end

	frame.parent = parent
	frame.MouseDown = false

	function frame.MouseButtonDown(fButton)
		if (fButton == fLeft) then
			frame.MouseDown = true
			frame.Drag = false
			local mouseData = Inspect.Mouse()
			frame.xDiff = frame.parent:GetLeft() - mouseData.x
			frame.yDiff = frame.parent:GetTop() - mouseData.y
			if frame.parent.Event.RightDown then frame.parent.Event.RightDown() end
		end
	end

	function frame.Event:LeftDown()
		self.MouseButtonDown(true)
		if self.parent.Event.LeftDown then self.parent.Event.LeftDown() end
	end

	function frame.Event:RightDown()
		self.MouseButtonDown(false)
		if self.parent.Event.RightDown then self.parent.Event.RightDown() end
	end
	
	function frame.Event:MouseMove(mouseX, mouseY)
		if self.MouseDown then
			self.Drag = true
			self.parent:SetPoint("TOPLEFT", UIParent, "TOPLEFT", mouseX + self.xDiff, mouseY + self.yDiff)
			UISettings[variable] = { x = mouseX + self.xDiff, y = mouseY + self.yDiff }
		end
		if self.parent.Event.MouseMove then self.parent.Event.MouseMove() end
	end
	
	function frame.Event:LeftUp()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.LeftUp then self.parent.Event.LeftUp() end
	end

	function frame.Event:LeftUpoutside()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.LeftUp then self.parent.Event.LeftUp() end
	end

	function frame.Event:RightUp()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.RightUp then self.parent.Event.RightUp() end
	end

	function frame.Event:RightUpoutside()
		self.MouseDown = false
		self.Drag = false
		if self.parent.Event.RightUp then self.parent.Event.RightUp() end
	end

	function frame.Event:LeftClick()
		if not self.Drag and self.parent.Event.LeftClick then self.parent.Event.LeftClick() end
	end

	function frame.Event:MouseIn()
		if self.parent.Event.MouseIn then self.parent.Event.MouseIn() end
	end

	function frame.Event:MouseOut()
		if self.parent.Event.MouseOut then self.parent.Event.MouseOut() end
	end
	return frame
end
