function RaidExtension.GUI.CreateFrame(parent, align, x, y, w, h, layer)
	local frame = UI.CreateFrame("Frame", "", parent)
	frame:SetPoint(align, parent, "TOPLEFT", x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)
	return frame
end

function RaidExtension.GUI.CreateText(parent, align, x, y, text, layer)
	local frame = UI.CreateFrame("Text", "", parent)
	frame:SetFontSize(14)
	frame:SetPoint(align, parent, "TOPLEFT", x, y)
	if text then frame:SetText(text) end
	if layer then frame:SetLayer(layer) end

	function frame.SetColor(color)
		frame:SetFontColor(color[1], color[2], color[3])
	end

	return frame
end

function RaidExtension.GUI.CreateIconicText(parent, align, x, y, w, h, fExt)
	local frame = RaidExtension.GUI.CreateFrame(parent, align, x, y, w, h, 0)
	frame.icon = RaidExtension.GUI.CreateTexture(frame, 1, 3, 15, 15, 0)
	frame.val = RaidExtension.GUI.CreateText(frame, "TOPRIGHT", w, 0)
	RaidExtension.GUI.CreateTexture(frame, 0, 2, 17, 17, 1, "img/icon-border.png")
	if fExt then frame.valExt = RaidExtension.GUI.CreateText(frame, "TOPLEFT", 16, 0) end
	return frame
end

function RaidExtension.GUI.CreateTexture(parent, x, y, w, h, layer, texture, layout)
	local frame = UI.CreateFrame("Texture", "", parent)
	frame:SetPoint(ifnull(layout, "TOPLEFT"), parent, ifnull(layout, "TOPLEFT"), x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)
	if texture then	frame:SetTexture(Addon.identifier, texture) end

	function frame.SetRect(x, y, w, h)
		frame:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
		frame:SetWidth(w)
		frame:SetHeight(h)
	end

	return frame
end

function RaidExtension.GUI.CreateRiftButton(parent, x, y, layer)
	local frame = UI.CreateFrame("RiftButton", "", parent)
	frame:SetPoint("TOPRIGHT", parent, "TOPRIGHT", x, y)
	frame:SetSkin("close")
	frame:SetLayer(layer)
	return frame
end

function RaidExtension.GUI.CreateButton(parent, x, y, w, h, layer, texture)
	local frame = UI.CreateFrame("Texture", "", parent)
	if x and y then frame:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y) end
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetLayer(layer)
	frame:SetTexture(Addon.identifier, "img/" .. texture .. "-normal.png")

	function frame.Event:LeftDown()
		frame:SetTexture(Addon.identifier, "img/" .. texture .. "-down.png")
	end

	function frame.Event:MouseIn()
		frame:SetTexture(Addon.identifier, "img/" .. texture .. "-hover.png")
	end

	function frame.Event:MouseMove()
		frame:SetTexture(Addon.identifier, "img/" .. texture .. "-hover.png")
	end

	function frame.Event:MouseOut()
		frame:SetTexture(Addon.identifier, "img/" .. texture .. "-normal.png")
	end
	return frame
end

function RaidExtension.GUI.CreateLink(parent, align, x, y, text, callback)
	local frame = UI.CreateFrame("Text", "", parent)
	frame:SetFontSize(14)
	frame:SetPoint(align, parent, "TOPLEFT", x, y)
	if text then frame:SetText(text) end
	frame.selected = false
	frame.hover = false

	function frame.Select(flag)
		frame.selected = flag
		frame.Update()
	end

	function frame.Update()
		local color = RaidExtension.GUI.LinkColors[iif(frame.selected, 3, 1) + iif(frame.hover, 1, 0)]
		frame:SetFontColor(color[1], color[2], color[3])
	end

	function frame.Event:MouseIn()
		frame.hover = true
		frame.Update()
	end

	function frame.Event:MouseOut()
		frame.hover = false
		frame.Update()
	end

	function frame.Event:LeftClick()
		if callback then callback() end
	end

	return frame
end
