function RaidExtension.Tooltip.Init(parent)
	RaidExtension.Tooltip.parent = parent
	RaidExtension.Tooltip.window = RaidExtension.GUI.CreateFrame(parent, "TOPLEFT", 0, 0, 320, 100, 100)
	RaidExtension.Tooltip.window:SetVisible(false)

	RaidExtension.GUI.CreateTexture(RaidExtension.Tooltip.window, 0, 0, 320, 12, 0, "img/tooltip-top.png", "TOPLEFT")
	RaidExtension.GUI.CreateTexture(RaidExtension.Tooltip.window, 0, 0, 320, 26, 0, "img/tooltip-bottom.png", "BOTTOMLEFT")
	RaidExtension.Tooltip.background = RaidExtension.GUI.CreateTexture(RaidExtension.Tooltip.window, 0, 12, 320, 50, 0, "img/tooltip-middle.png", "TOPLEFT")

	RaidExtension.Tooltip.fields = {
		classIcon = RaidExtension.GUI.CreateTexture(RaidExtension.Tooltip.background, 25, 5, 32, 32, 1),
		name = RaidExtension.GUI.CreateText(RaidExtension.Tooltip.background, "TOPLEFT", 65, 10),
		role = RaidExtension.GUI.CreateText(RaidExtension.Tooltip.background, "TOPRIGHT", 290, 10),
		captions = {},
		values = {}
	}
	RaidExtension.GUI.CreateTexture(RaidExtension.Tooltip.background, 5, 40, 320, 5, 0, "img/tooltip-sep.png", "TOPLEFT")

	RaidExtension.Tooltip.dynamic = {
		stats = RaidExtension.GUI.CreateFrame(RaidExtension.Tooltip.background, "TOPLEFT", 0, 50, 250, 100, 1),
		buffs = RaidExtension.GUI.CreateFrame(RaidExtension.Tooltip.background, "TOPLEFT", 0, 50, 250, 100, 1),
		rows = {}
	}

	for i = 1, 20 do
		local frame = RaidExtension.GUI.CreateFrame(RaidExtension.Tooltip.dynamic.buffs, "TOPLEFT", 35, 0, 250, 20, 0)
		frame.icon = RaidExtension.GUI.CreateTexture(frame, 1, 3, 15, 15, 1)
		RaidExtension.GUI.CreateTexture(frame, 0, 2, 17, 17, 2, "img/icon-border.png")
		frame.remaining = RaidExtension.GUI.CreateText(frame, "TOPLEFT", 20, 1)
		frame.description = RaidExtension.GUI.CreateText(frame, "TOPLEFT", 70, 1)
		table.insert(RaidExtension.Tooltip.dynamic.rows, frame)
	end
	RaidExtension.GUI.CreateTexture(RaidExtension.Tooltip.dynamic.buffs, 5, 125, 320, 5, 0, "img/tooltip-sep.png", "TOPLEFT")

	for viewType, viewStructure in pairs(RaidExtension.AE.Tooltip) do
		for k, v in ipairs(viewStructure) do
			if v.field then
				local frame = RaidExtension.GUI.CreateFrame(RaidExtension.Tooltip.dynamic[viewType], "TOPLEFT", 35, 0, 250, 20, 0)
				RaidExtension.Tooltip.fields.captions[viewType .. k] = RaidExtension.GUI.CreateText(frame, "TOPLEFT", iif(v.offset, 20, 0), v.pos)
				RaidExtension.Tooltip.fields.values[viewType .. k] = RaidExtension.GUI.CreateText(frame, "TOPRIGHT", iif(v.icon == nil, 250, 230), v.pos)
				if v.icon ~= nil then 
					RaidExtension.GUI.CreateTexture(frame, 236, v.pos + 3, 15, 15, 1, "img/" .. v.icon .. ".png")
					RaidExtension.GUI.CreateTexture(frame, 235, v.pos + 2, 17, 17, 2, "img/icon-border.png")
				end
			else
				RaidExtension.GUI.CreateTexture(RaidExtension.Tooltip.dynamic[viewType], 5, v.pos, 320, 5, 0, "img/tooltip-sep.png", "TOPLEFT")
			end
		end
	end
end

function RaidExtension.Tooltip.Show(id, x, y, data, fViewType)
	local height = 410
	local viewType = iif(fViewType, "stats", "buffs")
	RaidExtension.Tooltip.id = id

	RaidExtension.Tooltip.fields.classIcon:SetTexture(Addon.identifier, "img/class" .. data["props.class"] .. ".png")
	RaidExtension.Tooltip.fields.name:SetText(data.name)
	RaidExtension.Tooltip.fields.role:SetText(iif(data["props.role"], RaidExtension.AE.Roles[data["props.role"]], ""))

	RaidExtension.Tooltip.dynamic.stats:SetVisible(fViewType)
	RaidExtension.Tooltip.dynamic.buffs:SetVisible(not fViewType)

	for k, v in ipairs(RaidExtension.AE.Tooltip[viewType]) do
		if v.field then
			local resist = ifnull(data["data.stats.resistance.id"], "resist")
			local value = tostring(ifnull(data[v.field:gsub("{resistance}", resist)], ""))
			if v.resistance ~= nil and data[v.resistance] ~= resist then value = "0" end
			RaidExtension.Tooltip.fields.captions[viewType .. k]:SetText(tostring(v.caption:gsub("{resistance}", resist:gsub("resist", ""))))
			RaidExtension.Tooltip.fields.values[viewType .. k]:SetText(value)
			if v.class == nil then
				RaidExtension.Tooltip.fields.captions[viewType .. k]:SetVisible(true)
				RaidExtension.Tooltip.fields.values[viewType .. k]:SetVisible(true)
			else
				RaidExtension.Tooltip.fields.captions[viewType .. k]:SetVisible(v.class[data["props.class"] - 1])
				RaidExtension.Tooltip.fields.values[viewType .. k]:SetVisible(v.class[data["props.class"] - 1])
			end
		end
	end

	if not fViewType then
		local id = 1
		for k, v in ipairs({"food", "vial","enchant"}) do
			for bk, bv in ipairs(data["bufflist." .. v]) do
				RaidExtension.Tooltip.dynamic.rows[id]:SetPoint("TOPLEFT", RaidExtension.Tooltip.background, "TOPLEFT", 40, 170 + id * 20)
				RaidExtension.Tooltip.dynamic.rows[id].icon:SetTexture("Rift", bv.icon)
				RaidExtension.Tooltip.dynamic.rows[id].remaining:SetText(RaidExtension.Utils.GetTime(bv.remaining))
				RaidExtension.Tooltip.dynamic.rows[id].description:SetText(bv.description)
				id = id + 1
			end
		end
		for i = 1, 20 do RaidExtension.Tooltip.dynamic.rows[i]:SetVisible(i < id) end
		height = 220 + id * 20
	end
	RaidExtension.Tooltip.window:SetHeight(height)
	RaidExtension.Tooltip.background:SetHeight(height - 38)
	RaidExtension.Tooltip.window:SetPoint("TOPLEFT", RaidExtension.Tooltip.parent, "TOPLEFT", x, y - iif(y < UIParent:GetHeight() / 2, 0, height - 38))
	RaidExtension.Tooltip.window:SetVisible(true)
end

function RaidExtension.Tooltip.Hide(id)
	if RaidExtension.Tooltip.id == id then
		RaidExtension.Tooltip.window:SetVisible(false)
	end
end